library(shiny)

#runApp("~/Learn_Shiny/")

library(qvalue); library(MASS); library(matrixcalc); library(Matrix); library(gap); library(impute); library(EMMREML); library(gap)#; library(doParallel)

# Define server logic
shinyServer(function(input, output) {
  
  # wrapped in a call to renderPlot to indicate that:
  #  1) It is "reactive" and therefore should be automatically re-executed when inputs change
  #  2) Its output type is a plot
  
  #Function to simulate count data
  sim <- function(sites_max,mu_min, mu_max,covariate,pve,sigma2) {
    sigma2=3
    beta <- abs((pve*sigma2)/(pve-1))
    
    #Create that_j and theta_j2, the true methylation levels for each site
    if (input$test ==2) {
      #random effect and heritable component of the variance before negative logit link
      inFile2 <- input$vcv
      bb$covar_matrix <- as.matrix(read.delim(inFile2$datapath, header=F),ncol=input$n,nrow=input$n)
      if (is.positive.definite(bb$covar_matrix) == F) {
        as.matrix(nearPD(bb$covar_matrix)$mat) -> bb$covar_matrix
      }
      
      #read.delim(input$vcv, header=F) -> bb$covar_matrix
      theta_j <- Reduce(rbind,lapply(1:sites_max, function(sites) {
      e = rnorm(input$n, 0, sqrt(sigma2)*(1-input$h2))
      temp=as.matrix(sqrt(sigma2)*input$h2*bb$covar_matrix)
      u = mvrnorm(1, rep(0,input$n), temp) 
      theta_j <- covariate*beta + e + u
      return(t(theta_j))
    }))}
    else{
    theta_j <- Reduce(rbind,lapply(1:sites_max, function(sites) {
      e= rnorm(input$n, 0, sqrt(sigma2))
      theta_j <- covariate*beta + e
      #theta_j2 <- exp(theta_j)/(1 + exp(theta_j))
      return(t(theta_j))
    }))
    }
    theta_j2 <- exp(theta_j)/(1 + exp(theta_j))
    
    #Get matrix of counts per site per individual
    ##First get the average coverage
    cov <- runif(sites_max, min=mu_min, max=mu_max) #average coverage
        #neg_bin_param=neg_bin[sample(nrow(neg_bin), 1), ] #Alternate option if we provide a matrix of neg_binom parameters from real data. Would have to subset to get sites with appropriate coverage
    ##Then get simulated count data (sites-by-n matrix)
    counts <- Reduce(rbind,lapply(1:sites_max, function(sites) {
      coverage_n_j <- rnegbin(input$n,cov[sites],1.55223+0.04531*cov[sites])
      return(t(coverage_n_j))
    } ))
    
    #Get matrix of methylated sites per individual
    mcounts <- Reduce(cbind,lapply(1:input$n,function(indiv) {
      mcounts <- rbinom(sites_max,counts[,indiv],theta_j2[,indiv])
      return((mcounts))
    }))
    
    #observed_pve <- Reduce(rbind,lapply(1:sites_max, function(site) {
    #  pve <- summary(lm(theta_j2[site,]~covariate[,1]))$r.squared
    #  return(pve)
    #}))
    
    return(list(counts=counts,mcounts=mcounts)) }

  list() -> bb #sets up a list to move information into
  #n <- reactive({input$n})
  
  predict <- NULL
  predict <- reactive({
    input$goButton
    predictor <- NULL
    isolate(if (input$predictor == 1) {predictor <- matrix(0,ncol=1, nrow=input$n); predictor[1:input$pos] <- 1})
    isolate(if (input$predictor == 2) {predictor <- matrix(rnorm(input$n,input$pred_mean,input$pred_sd),ncol=1,nrow=input$n)})
    isolate(if (input$predictor == 3) {
      inFile <- input$predictor2
      predictor <- as.matrix(read.delim(inFile$datapath, header=F), ncol=1)
    }
    else {
    write.table(predictor, "simulated_predictor.txt", sep="\t")
    }
    )#writes predictor output as long as the predictor wasn't user defined
    if (input$goButton > 0) {predictor}
  })
  
  sim_n <- NULL
  sim_n <- reactive({sim((5000-5000*input$tp),input$coverage[1], input$coverage[2], predict(),0,sigma2)})
  sim_t <- NULL
  sim_t <- reactive({sim((5000*input$tp),input$coverage[1], input$coverage[2],predict(),input$pve,sigma2)})
  
  #Check if covariance matrix is positive definite
  output$PosDef <- renderText({
    input$goButton
    if (input$goButton == 0) {return()} #this prevents a graph from being created when the app first opens
    if (is.null(input$vcv)) {return()}
    if (input$test == 2) {
      inFile2 <- input$vcv
      vcv <- as.matrix(read.delim(inFile2$datapath, header=F),ncol=input$n,nrow=input$n)
      if (is.positive.definite(vcv) == F) {return('--Warning: covariance matrix is not positive definite. Covariance matrix was adjusted to be positive definite using the nearPD function (from the matrix package).')}
      #else {print('')}
    }
  })
  
  #Actually runs the simulation and makes the output file. Uses input information from the gui
  
  output$distPlot <- renderPlot({
    input$goButton
    if (input$goButton == 0) {return()} #this prevents a graph from being created when the app first opens
    
    predictor <- isolate(predict())
    sim_null <- isolate(sim_n())
    sim_true <- isolate(sim_t())
    
    mcounts= isolate(as.matrix(rbind(sim_null$mcounts,sim_true$mcounts))) #previously sim_out1, now mcounts
    counts= isolate(as.matrix(rbind(sim_null$counts,sim_true$counts))) #previously sim_out2, now counts
    colnames(mcounts) <- colnames(counts)
    write.table(mcounts, "simulated_mcounts.txt", sep="\t", quote=F)
    write.table(counts, "simulated_counts.txt", sep="\t", quote=F)
    ratio <- mcounts/counts
    mvalues <- isolate(log((ratio + 0.000001)/(1-ratio+0.000001), base = 2)) #logit link transformation to m-values
    data <- cbind(mcounts, counts)
    
    #clus <- makeCluster(detectCores())  # for however many cores are available
    #registerDoParallel(detectCores())
    #clusterExport(clus,varlist=c("predictor","mvalues", "input","bb"),envir=environment())
    
    isolate(if (input$test == 4) {pval<-isolate(apply(mvalues,1,function(x) t.test(x[predictor==0],x[predictor==1],paired=F)$p.value))})
    isolate(if (input$test == 1) {pval <- isolate(apply(X = mvalues,MARGIN = 1,FUN = function(x) summary(lm(x ~ predictor[,1]))$coefficients[2,4]))})
    isolate(if (input$test == 2) {
      pval <- vector(length=5000)
      
      inFile2 <- input$vcv
      vcv <- as.matrix(read.delim(inFile2$datapath, header=F),ncol=input$n,nrow=input$n)
      if (is.positive.definite(vcv) == F) {
        as.matrix(nearPD(vcv)$mat) -> vcv
      }
      mat2 <- matrix(0,nrow=input$n, ncol=input$n); for (i in 1:input$n) {mat2[i,i] <- 1}
      
      impute.knn(as.matrix(ratio))$data -> imp_ratio
      
      cbind(predictor,1) -> pred_int
      
    for (i in 1:5000) {
      pval[i] <- emmreml(y = (as.matrix(imp_ratio[i,], ncol=1)), X=pred_int, Z = vcv, K = mat2, varbetahat = T, varuhat = T, test = T)$pvalbeta[1,8]
      }
    })
    isolate(if (input$sure == TRUE & input$test == 3) {
      #Functions for the beta-binomial model from Xiang Zhou (http://www.xzlab.org/software/macau/bb.r)
      # fit beta-binomial models
      # alternative
      calc_logl_alt<-function(param) {
        phi=exp(param[1])/(exp(param[1])+1)
        mu=param[2]
        gamma=param[3]
        theta=exp(c*gamma+mu)/(exp(c*gamma+mu)+1)
        
        alpha=theta*(1-phi)/phi
        beta=(1-theta)*(1-phi)/phi
        
        logl=sum(lbeta(a = alpha+x, b = beta+y-x))-sum(lbeta(a = alpha, b = beta))
        return(logl)
      }
      # null
      calc_logl_null<-function(param) {
        phi=exp(param[1])/(exp(param[1])+1)
        mu=param[2]
        gamma=0
        theta=exp(mu)/(exp(mu)+1)
        
        alpha=theta*(1-phi)/phi
        beta=(1-theta)*(1-phi)/phi
        
        logl=sum(lbeta(alpha+x, beta+y-x))-length(x)*lbeta(alpha, beta)
        return(logl)
      }
      # fit the beta-binomial for each site, where mcounts is a vector of methylated counts, tcounts is a vector of total counts and cvt is a vector of covariates
      bbfit<-function (mcounts, tcounts, cvt) {
        x=mcounts; y=tcounts; c=cvt
        param=c(-1,0.5,0)
        phi=exp(param[1])/(exp(param[1])+1)
        mu=param[2]
        gamma=param[3]
        theta=exp(c*gamma+mu)/(exp(c*gamma+mu)+1)
        
        alpha=theta*(1-phi)/phi
        beta=(1-theta)*(1-phi)/phi
        
        phi=1/(alpha+beta+1); mu=log(alpha/beta); gamma=0;
        par=c(-2, mu)
        
        for (i_iter in 1:100) {
          if (i_iter==1) {
            null=optim(par,calc_logl_null, control=list(fnscale=-1),  method = c("CG"))
          } else {
            null=optim(alt$par[1:2],calc_logl_null, control=list(fnscale=-1),  method = c("CG"))
          }
          alt=optim(c(null$par,0),calc_logl_alt, control=list(fnscale=-1),  method = c("CG"))
          
          while (null$value==0 | alt$value==0 | alt$value<(null$value-0.01)) {
            alpha=runif(1); beta=runif(1)
            phi=1/(alpha+beta+1); mu=log(alpha/beta); gamma=0;
            par=c(log(phi), mu)
            
            null=optim(par,calc_logl_null, control=list(fnscale=-1),  method = c("CG"))
            alt=optim(c(null$par,0),calc_logl_alt, control=list(fnscale=-1),  method = c("CG"))
          }
          
          lr=2*(alt$value-null$value)
          if (lr<0) {lr=0}
          
          if (i_iter==1) {
            lr_old=lr
            lr_new=lr
          } else {
            lr_new=lr
            if (abs(lr_new-lr_old)<0.01 & lr!=0) {break}
            lr_old=lr
          }
        }
        
        return(1-pchisq(lr,df=1))
      }
      
      #Run beta binomial model for each site
      pval <- vector(length=5000); c <- unlist(predictor)
      for (k in 1:5000) {
        x <- unlist(mcounts[k,]); y <- unlist(counts[k,])
        pval[k] <- bbfit(mcounts = as.vector(x), tcounts = as.vector(y), cvt = as.vector(c))
        #print(k)
        }
      })
    isolate(if (input$sure == FALSE & input$test==3) {return()})
    
    #difference between conditions (if a binary predictor variable)
    mean_0 <- isolate(rowMeans(ratio[(5001-5000*input$tp):5000, predictor==0], na.rm=T)) 
    mean_1 <- isolate(rowMeans(ratio[(5001-5000*input$tp):5000, predictor==1], na.rm=T))
    dif <- mean_0-mean_1
    
    #output matrix of p and q values
    output <- isolate(matrix(ncol=2, nrow=5000))
    output[,1]<- isolate(pval)
    output[,2] <- isolate(as.matrix(qvalue(pval)$qvalues))
    #out2 refers to only the true positives (the last 5000*input$tp rows in the dataset)
    out2 <- isolate(output[(5001-5000*input$tp):5000,2]); out2 <- isolate(as.matrix(out2))
    #The number of true positives detected at the FDR threshold
    detected <- isolate(nrow(subset(out2, out2[,1] < input$fdr)))
    #The number of false positives detected at the FDR threshold
    false_pos <- isolate(nrow(subset(output, output[,2] < input$fdr)) - detected)
    
    write.table(output, "p_and_qvalues.txt", row.names=F, col.names=F, sep="\t", quote=F)
    
    #plot Q-Q plot
    par(mar=c(4,4,0,0))
    qqunif(output[,1], pch=16, ylim=c(0,ceiling(max(-log(output[,1], base=10)))), xlim=c(0,ceiling(max(-log(output[,1], base=10)))), frame.plot=F)
    
    isolate(
    if (input$predictor == 1)
        {legend("bottomright", legend=c(paste("Percent of true positives detected: ",isolate(detected/(50*input$tp)),"%"),paste("Percent of true negatives detected: ",isolate(round(false_pos/(50-50*input$tp), digits=5)),"%"), 
              #paste("PVE = ", round(mean(sim_true$output3[,1]), digits = 4)*100,"%"), 
              paste("Percent difference = ", round(abs(mean(dif)), digits=3)*100, "%")                    
        ), bty = 'n')}
    else {
      legend("bottomright", legend=c(paste("Percent of true positives detected: ",isolate(detected/(50*input$tp)),"%"),paste("Percent of true negatives detected: ",isolate(round(false_pos/(50-50*input$tp), digits=5)),"%") 
         #paste("PVE = ", round(mean(sim_true$output3[,1]), digits = 4)*100,"%")                    
      ), bty = 'n')}
    )
   }, height = 450, width=450
  )
  
})
